#include "burnint.h"
#include "gal.h"
// Galaxy Force HW - Run module

unsigned char GalGame=0; // 0x10=GF2, 0x20=PD, 0x30=RC, 0x40=G-Loc, 0x50=Strike Fighter
unsigned char GalDip[2]={0,0};
unsigned char GalReset=0;

static int DrvReset()
{
  int i=0;
  for (i=0;i<3;i++) { SekOpen(i); SekReset(); SekClose(); } // Reset cpus
  ZetReset(); // reset z80
  return 0;
}

// ----------------------------------------
int GalInit()
{
  int Ret=0;

  Ret=GalMemInit(); if (Ret!=0) return 1;

  GalSndInit();

  // ----------------- Draw init --------------------
  BsysPram=GalRamM19; BsysPalInit();
  GalLineSrc=GalRamM18+0x8000; // Line sprite tables in Cpu M Video ram
  GalSprSrc=GalRamX18+0x0000; // Normal sprite tables in Cpu X Video ram
  // ------------------------------------------------

  DrvReset(); // Reset machine

  return 0;
}

int GalExit()
{
  // ----------------- Draw exit --------------------
  BsysPalExit();
  // ------------------------------------------------

  GalSndExit();
  GalMemExit();
  return 0;
}

static INLINE int CheckSleep(int j)
{
// Galaxy Force loop points
#define PC_GF_M  (pc==0x1406e || pc==0x14072)
#define  L_GF_M  (((char)GalRamMFF[0xf000^1])>=0)
#define PC_GF_X1 (pc== 0xa0c4 || pc== 0xa0c8)
#define  L_GF_X1 (((char)GalRamXFF[0xf200^1])>=0)
#define PC_GF_X2 (pc>= 0x9fc4 || pc<= 0x9fd4)
#define  L_GF_X2 (0) //OLDTST (((char)Ram0C [0x000e^1])>=0)
#define PC_GF_Y  (pc== 0x1f70 || pc== 0x1f74)
#define  L_GF_Y  (((char)GalRamYFF[0xf000^1])>=0)

  if (GalGame==0x10)
  {
    unsigned int pc; pc=SekPc(j);
    // Galaxy Force II sleep/wake up:
         if (j==0) { if (PC_GF_M)  return L_GF_M; }
    else if (j==1) { if (PC_GF_X1) return L_GF_X1; else if (PC_GF_X2) return L_GF_X2; }
    else           { if (PC_GF_Y)  return L_GF_Y; }
    // End of Galaxy Force II sleep/wake up
  }

// Power Drift loop points
#define PC_PD_M  (pc==0x1188e || pc==0x11892)
#define  L_PD_M  (GalRamMFF[0xf9f0^1])
#define PC_PD_X  (pc==0xdb90 || pc==0xdb96)
#define  L_PD_X  (GalRam0C[0xff0c^1])
#define PC_PD_Y  (pc==0x33084 || pc==0x3308a)
#define  L_PD_Y  (((char)GalRam0C[0xff0e^1])>=0)

  if (GalGame==0x20)
  {
    unsigned int pc; pc=SekPc(j);
    // Power Drift sleep/wake up:
         if (j==0) { if (PC_PD_M) return L_PD_M; }
    else if (j==1) { if (PC_PD_X) return L_PD_X; }
    else           { if (PC_PD_Y) return L_PD_Y; }
    // End of Power Drift sleep/wake up
  }
  return 0;
}

static INLINE int GearDisplay()
{
  // Display the current gear on the screen
  static unsigned char ps[16]={0x68,0x70,0xef,0,0x10,0,0,0,0x2b,0x03,0,0,0,0,0,0};
  // Setup up the BsysSpr for the Line sprites:
  BsysSprData=GalLineData; BsysSprDataLen=GalLineDataLen;
  ps[0]=216; ps[1]=216+8;  ps[3]=0x00; ps[2]=0xb8; // Y,X
  if (GalButton[5]) { ps[6]=0xa0; ps[7]=0xad; } // 75b60  low gear
  else              { ps[6]=0x20; ps[7]=0xae; } // 75c60 high gear
  GalLineDrawSprite(0,ps);
  BsysSprData=NULL; BsysSprDataLen=0;
  return 0;
} 

// Draw graphics
static int DrvDraw()
{
  int SkyOff=0;
  BsysPalUpdate(); // Update the palette

  SkyOff=0x88;
  if ((GalGame&0xf0)==0x20) SkyOff=0x30; // Power Drift
  if ((GalGame&0xe0)==0x40) SkyOff=0x30; // G-Loc/Strike Fighter

  {
    // sky
    unsigned char *pl=NULL; int y=0;
    for (pl=pBurnDraw,y=0; y<224; pl+=nBurnPitch,y++)
    { unsigned int c; c=BsysPal[SkyOff+y]; BurnFillLine(pl,c); }
  }

  GalSprDraw();
  GalLineDraw();
  if ((GalGame&0xf0)==0x20) GearDisplay(); // show power drift gear state

  return 0;
}

// Do one frame
int GalFrame()
{
  int Done[4],Total[4],i=0,Interleve=100;
  if (GalReset) DrvReset(); // Reset machine

  if ((GalGame&0xf0)==0x20)
  {
    if (GalGear && GalLastGear==0) GalButton[5]^=1; // Toggle low/high gear
    GalLastGear=GalGear;
  }

  // Compile separate buttons into GalInput
  GalInput=0; for (i=0;i<8;i++) GalInput|=(GalButton[i]&1)<<i;

  // Make Analog values to return via address 100000
  for (i=0;i<GAL_ANALOG_COUNT;i++) GalAnaVal[i]=GalAnalog[i];

  GalInputPrepare();

  Total[0]=Total[1]=Total[2]=12000000/60; // 12mhz 68000
  Total[3]=4000000/60; // 4mhz Z80
  Done[0]=Done[1]=Done[2]=Done[3]=0;

  AftSoundFrameStart(); // start rendering this frames's sound

  SekOpen(0); SekInterrupt(4); SekRun(1000); SekInterrupt(2); SekRun(500); SekClose();
  SekOpen(1); SekInterrupt(4); SekRun(1000); SekInterrupt(2); SekRun(500); SekClose();
  SekOpen(2); SekInterrupt(4); SekRun(1000); SekInterrupt(2); SekClose();

  for (i=0;i<Interleve;i++)
  {
    int j=0,Next=0; // Next cycle count to run to
    int k=Interleve-i-1;

    GalInputMemory();

    // Run all 68000s
    for (j=0;j<3;j++)
    {
      int Sleep;
      Sleep=CheckSleep(j);
      if (Sleep) continue; // See if this cpu can sleep
      SekOpen(j);
      Next=(i+1)*Total[j]/Interleve; // find out next cycle count to run to
      Done[j]+=SekRun(Next-Done[j]);  // run cpu
      SekClose();
    }

    // Interleve the z80 a little bit (make sure we do it all though: hence 'k&15' not 'i&15' )
    j=3;
    if ((k&15)==0)
    {
      int Exe;
      Next=(i+1)*Total[j]/Interleve; // find out next cycle count to run to
      Exe=ZetRun(Next-Done[j]);   // run cpu
      Done[j]+=Exe;
      nAftFmTimer-=Exe; // Move timer timeout back
      AftSoundRenderTo((i+1)*nBurnSoundLen/Interleve); // Render sound for this portion
    }
  }

  AftSoundFrameStop(); // stop rendering this frames's sound

  if (pBurnDraw!=NULL) DrvDraw(); // Draw screen if needed
  return 0;
}
